@@ -4,10 +4,10 @@
 
 from __future__ import annotations
 
-import numbers
 import pickle
 import sys
-from typing import Any, Callable, List, Tuple, Type, TypeVar, Union, cast
+from pathlib import Path
+from typing import Any, Callable, List, Tuple, Type, Union, cast
 
 import pytest
 
@@ -30,13 +30,18 @@ from mbedtls.pk import (
     get_supported_curves,
 )
 
+if sys.version_info < (3, 11):
+    from typing_extensions import assert_never
+else:
+    from typing import assert_never
+
 if sys.version_info < (3, 8):
     from typing_extensions import Literal
 else:
     from typing import Literal
 
 
-T = TypeVar("T")
+_CipherType = Union[RSA, ECC]
 
 
 def test_supported_curves() -> None:
@@ -74,12 +79,12 @@ def test_rsa_encryp_decrypt(randbytes: Callable[[int], bytes]) -> None:
     assert rsa.decrypt(rsa.encrypt(msg)) == msg
 
 
-def do_generate(cipher: T) -> bytes:
+def do_generate(cipher: _CipherType) -> bytes:
     if isinstance(cipher, RSA):
         return cipher.generate(1024)
     if isinstance(cipher, ECC):
         return cipher.generate()
-    pytest.fail("unreachable")
+    assert_never(cipher)
 
 
 class TestECPoint:
@@ -142,63 +147,110 @@ class TestCipher:
             ],
         )
     )
-    def cipher(self, request: Any) -> Union[ECC, RSA]:
+    def cipher(self, request: Any) -> _CipherType:
         if request.param is RSA:
             return cast(RSA, request.param())
         return ECC(request.param)
 
-    def test_pickle(self, cipher: Union[ECC, RSA]) -> None:
+    def test_pickle(self, cipher: _CipherType) -> None:
         assert cipher == pickle.loads(pickle.dumps(cipher))
 
-    def test_hash(self, cipher: Union[ECC, RSA]) -> None:
+    def test_hash(self, cipher: _CipherType) -> None:
         assert isinstance(hash(cipher), int)
 
-    def test_export_private_key(self, cipher: Union[ECC, RSA]) -> None:
+    def test_export_private_key(
+        self, cipher: _CipherType, tmp_path: Path
+    ) -> None:
+        cipher_tag = {RSA: "RSA", ECC: "EC"}[type(cipher)]
+
         assert cipher.export_key("DER") == b""
         assert cipher.export_key("PEM") == ""
         assert cipher.key_size == 0
 
         der = do_generate(cipher)
-
         assert der
-        assert cipher.export_key() == der
-        assert cipher.export_key("DER") == der
-        assert cipher.export_key("PEM") != ""
-        assert cipher == cipher.export_key("DER")
-        assert cipher == cipher.export_key("PEM")
-        assert type(cipher).from_DER(cipher.export_key("DER")) == cipher
-        assert type(cipher).from_PEM(cipher.export_key("PEM")) == cipher
         assert cipher.key_size > 0
 
-    def test_import_private_key(self, cipher: Union[ECC, RSA]) -> None:
+        assert der == cipher.export_key()
+        assert der == cipher.export_key("DER")
+        assert der == bytes(cipher)
+        assert der == cipher
+        assert cipher == type(cipher).from_DER(der)
+
+        pem = cipher.export_key("PEM")
+        assert pem.startswith(f"-----BEGIN {cipher_tag} PRIVATE KEY-----\n")
+        assert pem.endswith(f"-----END {cipher_tag} PRIVATE KEY-----\n")
+        assert pem == str(cipher)
+        assert pem == cipher.to_PEM().private
+        assert cipher == type(cipher).from_PEM(pem)
+
+    @pytest.mark.parametrize(
+        "copy",
+        [
+            lambda cipher: type(cipher).from_DER(cipher.export_key("DER")),
+            lambda cipher: type(cipher).from_PEM(cipher.export_key("PEM")),
+        ],
+    )
+    def test_import_private_key(
+        self, cipher: _CipherType, copy: Callable[[_CipherType], _CipherType]
+    ) -> None:
         assert not cipher.export_key()
         assert not cipher.export_public_key()
 
         key = do_generate(cipher)
+        assert key
 
+        other = copy(cipher)
         other = type(cipher).from_buffer(key)
-        assert other.export_key()
-        assert other.export_public_key()
+        assert other == cipher
+        assert other.export_key() == cipher.export_key() == key
+        assert other.export_public_key() == cipher.export_public_key()
         assert check_pair(cipher, other) is True  # Test private half.
         assert check_pair(other, cipher) is True  # Test public half.
         assert check_pair(other, other) is True
-        assert cipher == other
 
-    def test_export_public_key(self, cipher: Union[ECC, RSA]) -> None:
+    def test_import_from_file(
+        self, cipher: _CipherType, tmp_path: Path
+    ) -> None:
+        do_generate(cipher)
+
+        prv_der_file = tmp_path / "crt.prv.der"
+        prv_der_file.write_bytes(cipher.export_key("DER"))
+        assert cipher == type(cipher).from_file(prv_der_file)
+        prv_der_file.unlink()
+
+        pub_der_file = tmp_path / "crt.pub.der"
+        pub_der = cipher.export_public_key("DER")
+        pub_der_file.write_bytes(pub_der)
+        assert pub_der == type(cipher).from_file(pub_der_file)
+        pub_der_file.unlink()
+
+        prv_pem_file = tmp_path / "crt.prv.pem"
+        prv_pem_file.write_text(cipher.export_key("PEM"))
+        assert cipher == type(cipher).from_file(prv_pem_file)
+        prv_pem_file.unlink()
+
+        pub_pem_file = tmp_path / "crt.pub.pem"
+        pub_pem = cipher.export_public_key("PEM")
+        pub_pem_file.write_text(pub_pem)
+        assert pub_pem == type(cipher).from_file(pub_pem_file)
+        pub_pem_file.unlink()
+
+    def test_export_public_key(self, cipher: _CipherType) -> None:
         assert cipher.export_public_key("DER") == b""
         assert cipher.export_public_key("PEM") == ""
 
-        der = do_generate(cipher)
-
+        do_generate(cipher)
+        der = cipher.export_public_key("DER")
         assert der
-        assert type(cipher).from_DER(
-            cipher.export_public_key("DER")
-        ) == cipher.export_public_key("DER")
-        assert type(cipher).from_PEM(
-            cipher.export_public_key("PEM")
-        ) == cipher.export_public_key("PEM")
+        assert der == type(cipher).from_DER(der).export_public_key("DER")
 
-    def test_import_public_key(self, cipher: Union[ECC, RSA]) -> None:
+        pem = cipher.export_public_key("PEM")
+        assert pem.startswith("-----BEGIN PUBLIC KEY-----\n")
+        assert pem.endswith("-----END PUBLIC KEY-----\n")
+        assert pem == type(cipher).from_PEM(pem).export_public_key("PEM")
+
+    def test_import_public_key(self, cipher: _CipherType) -> None:
         assert not cipher.export_key()
         assert not cipher.export_public_key()
 
@@ -220,7 +272,7 @@ class TestCipher:
     )
     def test_sign_verify(
         self,
-        cipher: Union[ECC, RSA],
+        cipher: _CipherType,
         digestmod: str,
         randbytes: Callable[[int], bytes],
     ) -> None:
@@ -248,7 +300,24 @@ class TestECCExportKey:
         ecc.generate()
         assert ecc.export_key("NUM") != 0
 
-    def test_export_public_key_to_point(self, curve: Curve) -> None:
+        if curve in (Curve.CURVE25519, Curve.CURVE448):
+            with pytest.raises(ValueError):
+                ecc.export_key("DER")
+        else:
+            der = ecc.export_key("DER")
+            assert der
+            assert ECC(curve).from_DER(der) == der
+
+        if curve in (Curve.CURVE25519, Curve.CURVE448):
+            with pytest.raises(ValueError):
+                ecc.export_key("PEM")
+        else:
+            pem = ecc.export_key("PEM")
+            assert pem
+            assert pem.startswith("-----BEGIN EC PRIVATE KEY-----\n")
+            assert pem.endswith("-----END EC PRIVATE KEY-----\n")
+
+    def test_export_public(self, curve: Curve) -> None:
         ecc = ECC(curve)
         assert ecc.export_public_key("POINT") == 0
         assert ecc.export_public_key("POINT") == ECPoint(0, 0, 0)
@@ -257,18 +326,33 @@ class TestECCExportKey:
 
         pub = ecc.export_public_key("POINT")
         assert isinstance(pub, ECPoint)
-        assert isinstance(pub.x, numbers.Integral)
-        assert isinstance(pub.y, numbers.Integral)
-        assert isinstance(pub.z, numbers.Integral)
         assert pub.x not in (0, pub.y, pub.z)
         if curve in (Curve.CURVE25519, Curve.CURVE448):
             assert pub.y == 0
         else:
             assert pub.y not in (0, pub.x, pub.z)
-        assert pub.z == 0 or pub.z == 1
+        assert pub.z in (0, 1)
+
+        if curve in (Curve.CURVE25519, Curve.CURVE448):
+            with pytest.raises(ValueError):
+                ecc.export_public_key("DER")
+        else:
+            der = ecc.export_public_key("DER")
+            assert der
+
+        if curve in (Curve.CURVE25519, Curve.CURVE448):
+            with pytest.raises(ValueError):
+                ecc.export_public_key("PEM")
+        else:
+            pem = ecc.export_public_key("PEM")
+            assert pem
+            assert pem.startswith("-----BEGIN PUBLIC KEY-----\n")
+            assert pem.endswith("-----END PUBLIC KEY-----\n")
 
 
 class TestECCtoECDH:
+    # pylint: disable=protected-access
+
     @pytest.mark.parametrize("curve", get_supported_curves())
     def test_exchange(self, curve: Curve) -> None:
         ecp = ECC(curve)
@@ -330,6 +414,8 @@ class TestDH:
 
 
 class TestECDH:
+    # pylint: disable=protected-access
+
     @pytest.fixture(params=get_supported_curves())
     def key(self, request: Any) -> ECC:
         key = ECC(request.param)
@@ -363,10 +449,10 @@ class TestECDH:
 
     def test_client_accessors_with_key(self, key: ECC) -> None:
         der = key.generate()
-        format: Literal["NUM", "DER"] = (
+        fmt: Literal["NUM", "DER"] = (
             "NUM" if key.curve in (Curve.CURVE25519, Curve.CURVE448) else "DER"
         )
-        assert der == key.export_key(format)
+        assert der == key.export_key(fmt)
 
         peer = ECDHClient(key)
 
@@ -380,10 +466,10 @@ class TestECDH:
 
     def test_server_accessors_with_key(self, key: ECC) -> None:
         der = key.generate()
-        format: Literal["NUM", "DER"] = (
+        fmt: Literal["NUM", "DER"] = (
             "NUM" if key.curve in (Curve.CURVE25519, Curve.CURVE448) else "DER"
         )
-        assert der == key.export_key(format)
+        assert der == key.export_key(fmt)
 
         peer = ECDHServer(key)
 
@@ -437,6 +523,8 @@ def do_exchange(alice: ECDHNaive, bob: ECDHNaive) -> None:
 
 
 class TestECDHNaive:
+    # pylint: disable=protected-access
+
     @pytest.fixture(params=[Curve.CURVE448, Curve.CURVE25519])
     def curve(self, request: Any) -> Curve:
         assert isinstance(request.param, Curve)
