@@ -43,6 +43,7 @@ from mbedtls.tls import (
     DTLSVersion,
     HandshakeStep,
     HelloVerifyRequest,
+    MaxFragmentLength,
     NextProtocol,
     Purpose,
     ServerContext,
@@ -440,6 +441,14 @@ class TestConfiguration:
         assert trust_store
         assert_conf_invariant(conf, trust_store=trust_store)
 
+    @pytest.mark.parametrize("mfl", MaxFragmentLength)
+    def test_max_fragmentation_length(
+        self,
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+        mfl: MaxFragmentLength,
+    ) -> None:
+        assert_conf_invariant(conf, max_fragmentation_length=mfl)
+
     def test_set_sni_callback(
         self,
         conf: Union[TLSConfiguration, DTLSConfiguration],
@@ -818,6 +827,33 @@ class TestDTLSHandshake:
         assert do_send(secret, src=client, dst=server) == secret
         assert do_send(secret, src=server, dst=client) == secret
 
+    @pytest.mark.parametrize("mtu_cli", [0, 128, 380, 500, (1 << 16) - 1])
+    @pytest.mark.parametrize("mtu_srv", [0, 128, 380, 500, (1 << 16) - 1])
+    def test_psk_set_mtu(self, mtu_cli: int, mtu_srv: int) -> None:
+        psk = ("cli", b"secret")
+        server = ServerContext(
+            DTLSConfiguration(
+                pre_shared_key_store=dict((psk,)),
+                validate_certificates=False,
+            )
+        ).wrap_buffers()
+        server.setmtu(mtu_srv)
+        client = ClientContext(
+            DTLSConfiguration(
+                pre_shared_key=psk,
+                validate_certificates=False,
+            ),
+        ).wrap_buffers("hostname")
+        client.setmtu(mtu_cli)
+        make_hello_verify_request(
+            client=client, server=server, cookie="🍪🍪🍪".encode()
+        )
+        make_full_handshake(client=client, server=server)
+
+        secret = b"a very secret message"
+        assert do_send(secret, src=client, dst=server) == secret
+        assert do_send(secret, src=server, dst=client) == secret
+
     def test_resume_from_pickle(self) -> None:
         psk = ("cli", b"secret")
         server = ServerContext(
@@ -1024,10 +1060,10 @@ class TestProgramsDTLS:
             "--psk-store",
             "cli=secret",
         ]
-        proc = subprocess.Popen(args, text=True, encoding="utf8")
-        yield proc
-        proc.kill()
-        proc.wait(1.0)
+        with subprocess.Popen(args, text=True, encoding="utf8") as proc:
+            yield proc
+            proc.kill()
+            proc.wait(1.0)
 
     @pytest.mark.repeat(3)
     @pytest.mark.usefixtures("server")
